unit fFunc;//GLScene

interface

uses Windows, Forms, Classes, Graphics, Controls, ExtCtrls,
	GLScene, GLObjects, GLWin32Viewer, GLCadencer, GLTexture;

type
	TfrmFunc = class(TForm)
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    function GetFuncType: Integer;
  protected
		procedure ROTATE_SCENE_OBJECT(O: TGLBaseSceneObject; Angle, newTime: Double; RTP: Byte=7);
  protected
    FTimer		: TTimer;
  	FScene		: TGLScene;
  	FViewer		: TGLSceneViewer;
  	FCamera		: TGLCamera;
    FLight		: TGLLightSource;
    FDummy		: TGLDummyCube;
    FCadencer	: TGLCadencer;
    procedure CREATE_OBJECT; virtual;
    procedure DESTROY_OBJECT; virtual;
    procedure SETUP_USER_INTERFACE; virtual;
    procedure DO_Timer(Sender: TObject); virtual;
    procedure DO_Resize(Sender: TObject); virtual;
    procedure DO_Close(Sender: TObject; var Action: TCloseAction); virtual;
    procedure DO_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
    procedure DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
		procedure DO_KeyPress(Sender: TObject; var Key: Char); virtual;
    procedure DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); virtual;
    procedure DO_MouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean); virtual;
    procedure DO_MouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean); virtual;

    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); virtual;

    procedure DO_ViewerClick(Sender: TObject); virtual;
    procedure DO_ViewerAfterRender(Sender: TObject); virtual;
    procedure DO_ViewerMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); virtual;
    procedure DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo); virtual;
    procedure DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double); virtual;

  public
    mx, my: Integer;
  	constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure PASTE_TO_MAIN; virtual;
    procedure PASTE_TO_OWNER; virtual;
    procedure WAIT_FOR_CADENCER; virtual;
    property FuncType: Integer read GetFuncType;
  end;

implementation

{$R *.dfm}

uses Math, SysUtils;

{ TfrmFunc }

procedure TfrmFunc.DESTROY_OBJECT;begin{}end;
procedure TfrmFunc.DO_ViewerClick(Sender: TObject);begin{}end;
procedure TfrmFunc.DO_ViewerAfterRender(Sender: TObject);begin{}end;
procedure TfrmFunc.DO_KeyPress(Sender: TObject; var Key: Char);begin{}end;
procedure TfrmFunc.DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);begin{}end;
procedure TfrmFunc.DO_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);begin{}end;
procedure TfrmFunc.DO_MouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);begin{}end;
procedure TfrmFunc.DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo);begin{}end;
procedure TfrmFunc.DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double);begin{}end;
procedure TfrmFunc.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
	if Shift <> [] then
  begin
  				if Shift = [ssLeft] 	then FCamera.MoveAroundTarget(my-Y, mx-X)
  	else 	if Shift = [ssRight] 	then FCamera.RotateTarget(my-Y, mx-X);
    mx := X;
    my := Y;
  end;
end;

procedure TfrmFunc.DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mx := X;
  my := Y;
end;

function TfrmFunc.GetFuncType: Integer;
begin
	Result := StrToInt(Copy(ClassName, 9, 5));
end;

procedure TfrmFunc.CREATE_OBJECT;
begin
  FTimer 		:= TTimer.Create(Self);
  FCadencer := TGLCadencer.Create(nil);
	FViewer 	:= TGLSceneViewer.Create(Self);
	FScene  	:= TGLScene.Create(Self);
  FCamera 	:= TGLCamera.Create(Self);
	FLight  	:= TGLLightSource.Create(Self);
  FDummy  	:= TGLDummyCube.Create(Self);

  with FViewer do
  begin
  	Align := alClient;
  	Buffer.BackgroundColor := clBlack;
    OnClick 		:= DO_ViewerClick;
    OnMouseUp 	:= DO_ViewerMouseUp;
    OnMouseDown := DO_ViewerMouseDown;
    OnMouseMove := DO_ViewerMouseMove;
    AfterRender := DO_ViewerAfterRender;
  end;

  with FTimer do
  begin
  	Enabled := False;
  	Interval := 1000;
    OnTimer := DO_TIMER;
  end;

  with FCadencer do
  begin
  	SleepLength := 35;
  	Scene := FScene;
  	Enabled := False;
	  TimeReference := cmPerformanceCounter;
  	OnProgress := DO_CadencerProgress;
  end;

  with FDummy do
  begin
  	OnProgress := DO_DummyProgress;
  end;

	OnClose := DO_Close;
	OnResize := DO_Resize;
	OnKeyDown := DO_KeyDown;
  OnKeyUp := DO_KeyUp;
  OnKeyPress := DO_KeyPress;
	OnMouseWheel := DO_MouseWheel;
  OnMouseWheelDown := DO_MouseWheelDown;
  OnMouseWheelUp := DO_MouseWheelUp;

	FCadencer.Enabled := True;
  FTimer.Enabled := True;
end;

procedure TfrmFunc.PASTE_TO_MAIN;
begin
  FViewer.Parent := TWinControl(Owner);
end;

procedure TfrmFunc.PASTE_TO_OWNER;
begin
  FViewer.Parent := TWinControl(Self);
end;

procedure TfrmFunc.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	if Assigned(FViewer) then FViewer.Invalidate;
end;

procedure TfrmFunc.DO_Resize(Sender: TObject);
begin
	if Assigned(FViewer) then FViewer.ResetPerformanceMonitor;
end;

procedure TfrmFunc.DO_Timer(Sender: TObject);
const
	FMT: string = 'Func%5.5d:  %.1f FPS';
begin
	if Assigned(FViewer) then
  begin
		TForm(Owner).Caption := Format(FMT, [FuncType, FViewer.FramesPerSecond]);
		FViewer.ResetPerformanceMonitor;
  end;
end;

constructor TfrmFunc.Create(AOwner: TComponent);
begin
  inherited;
  CREATE_OBJECT;
  SETUP_USER_INTERFACE;
  PASTE_TO_MAIN;
  FViewer.Camera := FCamera;
end;

destructor TfrmFunc.Destroy;
begin
	DESTROY_OBJECT;
  inherited;
end;

procedure TfrmFunc.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WAIT_FOR_CADENCER;
	PASTE_TO_OWNER;
  Action := caFree;
end;

procedure TfrmFunc.WAIT_FOR_CADENCER;
begin
  if Assigned(FCadencer) then
  begin
    FCadencer.Enabled := False;
    FCadencer.OnProgress := nil;
    FCadencer.Scene := nil;
    FreeAndNil(FCadencer);
  end;
end;

procedure TfrmFunc.DO_Close(Sender: TObject; var Action: TCloseAction);
begin
  WAIT_FOR_CADENCER;
	PASTE_TO_OWNER;
  Action := caFree;
end;

procedure TfrmFunc.SETUP_USER_INTERFACE;
begin

end;

procedure TfrmFunc.ROTATE_SCENE_OBJECT(O: TGLBaseSceneObject; Angle, newTime: Double; RTP: Byte);
begin
	if Assigned(O) then
		with O do
  	begin
			if (RTP and $01) = 1 then PitchAngle := Angle * newTime;
  		if (RTP and $02) = 2 then TurnAngle  := Angle * newTime;
	  	if (RTP and $04) = 4 then RollAngle  := Angle * newTime;
  	end;
end;

procedure TfrmFunc.DO_MouseWheelDown(Sender: TObject; Shift: TShiftState;
	MousePos: TPoint; var Handled: Boolean);
begin
end;

procedure TfrmFunc.DO_MouseWheel(Sender: TObject; Shift: TShiftState;
	WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var
	DistanceRatio: Single;
begin
  DistanceRatio := Power(1.1, WheelDelta/120);
	FCamera.AdjustDistanceToTarget(DistanceRatio);
  Handled := True;
end;

procedure TfrmFunc.DO_ViewerMouseUp(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
end;



end.
